@@ -105,6 +105,12 @@ G_BEGIN_DECLS
  * String to access property #DbusmenuClient:text-direction
  */
 #define DBUSMENU_CLIENT_PROP_TEXT_DIRECTION "text-direction"
+/**
+ * DBUSMENU_CLIENT_PROP_GROUP_EVENTS:
+ *
+ * String to access property #DbusmenuClient:group-events
+ */
+#define DBUSMENU_CLIENT_PROP_GROUP_EVENTS "group-events"
 
 /**
  * DBUSMENU_CLIENT_TYPES_DEFAULT:
@@ -212,7 +218,7 @@ gboolean             dbusmenu_client_add_type_handler_full (DbusmenuClient * cli
                                                         GDestroyNotify destroy_func);
 DbusmenuTextDirection dbusmenu_client_get_text_direction (DbusmenuClient * client);
 DbusmenuStatus       dbusmenu_client_get_status        (DbusmenuClient * client);
-const GStrv          dbusmenu_client_get_icon_paths    (DbusmenuClient * client);
+GStrv                dbusmenu_client_get_icon_paths    (DbusmenuClient * client);
 
 /**
 	SECTION:client
